@@ -276,10 +276,11 @@ load_host_cert(enum ipsec_cert_type certtype, const char *filename,
 
     if (*filename == '/')	/* absolute pathname */
     	strncpy(path, filename, ASN1_BUF_LEN);
-    else 			/* relative pathname */
-	snprintf(path, ASN1_BUF_LEN, "%s/%s", 
-		(!oco->certs_dir ? "." : oco->certs_dir)
-		,filename);
+    else if(oco->certs_dir == NULL) /* pathname relative to cwd */
+	snprintf(path, ASN1_BUF_LEN, "./%s", filename);
+    else 			/* pathname relative to rootdir */
+	snprintf(path, ASN1_BUF_LEN, "%s%s/%s", oco->rootdir
+		 , oco->certs_dir, filename);
 
     return load_cert(certtype, path, verbose, "host cert", cert);
 }
