@@ -196,12 +196,6 @@ extern bool do_command_freebsd(struct connection *c, struct spd_route *sr
 extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
 #endif
 
-#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
-extern bool do_command_darwin(struct connection *c, struct spd_route *sr
-			       , const char *verb, struct state *st);
-extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
-#endif
-
 #if defined(__CYGWIN32__)
 extern bool do_command_cygwin(struct connection *c, struct spd_route *sr
 			      , const char *verb, struct state *st);
@@ -333,6 +327,8 @@ extern bool eroute_connection(struct spd_route *sr
 extern const struct kernel_ops klips_kernel_ops;
 extern const struct kernel_ops mast_kernel_ops;
 
+extern bool kernel_overlap_supported(void);
+extern const char *kernel_if_name(void);
 extern void show_kernel_interface(void);
 
 #define _KERNEL_H_
