@@ -168,6 +168,8 @@ struct state
     int                st_usage;
 
     bool               st_ikev2;             /* is this an IKEv2 state? */
+    bool               st_rekeytov2;         /* true if this IKEv1 is about
+					      * to be replaced with IKEv2 */
 
     struct connection *st_connection;        /* connection for this SA */
     int                st_whack_sock;        /* fd for our Whack TCP socket.
